#include "common.hpp"

// System includes
#include <vector>

using std::vector;

namespace prc {

/**
 * Converts a byte vector representing an integer to its integer value.
 * The vector must contain sizeof(int) elements.
 * @param arr The vector<byte> to convert.
 * @return Returns the integer value of the byte array.
 */
int byteVectorToInt(vector<byte>& arr) {
	int num = 0;

	// NOTE: arr must contain at least sizeof(int) elements.
	for (unsigned int ii = 0; ii < sizeof(int); ii++) {
		num <<= CHAR_BIT;
		num |= arr[ii];
	}

	/*
	// This works too
	for (unsigned int ii = sizeof(int); ii < sizeof(int); ii++)
		num = (num << CHAR_BIT) + arr[ii];
	*/

	return num;
}

/**
 * Converts an integer to a byte vector.
 * @param num The integer to convert.
 * @return Returns a statically allocated vector<byte> that contains the byte 
 * representation of the given int.
 */
vector<byte> intToByteVector(int num) {
	vector<byte> data(sizeof(int));

	/*
	arr[0] = num >> 24;
        arr[1] = (num >> 16) & 0xFF;
        arr[2] = (num >> 8) & 0xFF;
        arr[3] = num & 0xFF;
	*/

	// This is a more concise way of doing the above.
	for (unsigned int ii = 0; ii < sizeof(int); ii++) {
		data[ii] = (num >> (sizeof(int) - ii - 1) * CHAR_BIT) & 0xFF;
	}

	return data;
}

} // end namespace prc
