#ifndef _RPC_HANDLER_HPP
#define _RPC_HANDLER_HPP

// XML-RPC includes
#include <xmlrpc-c/registry.hpp>

namespace prc {

/**
 * This handler is called when the project client sends a new work unit to the science 
 * application to be computed.  This method will compute the result of the work unit, 
 * then return that result to the project client.
 */
class ComputeResultHandler : public xmlrpc_c::method {
	/**
	 * This method is called when the sciapp.computeResult RPC is handled.
	 * @param paramList List of RPC parameters.
	 * @param retvalP Pointer to the return value of the method.
	 */
	public:
		void execute(xmlrpc_c::paramList const&,xmlrpc_c::value* const);
};

/**
 * This handler is called when the project server sends a spot check work unit to the science 
 * application to be computed.  This method will compute the result of the work unit, 
 * then return that result to the project server.  Checkpointing should not be used in this case 
 * because there is no project client involved in the interaction between the server and science 
 * application.  Note that only the client implements the checkpointing RPCs.
 */
class ComputeSpotCheckResultHandler : public xmlrpc_c::method {
	public:
		/**
		 * This method is called when the sciapp.computeSpotCheckResult RPC is handled.
		 * @param paramList List of RPC parameters.
		 * @param retvalP Pointer to the return value of the method.
	 	*/
		void execute(xmlrpc_c::paramList const&,xmlrpc_c::value* const);
};

/**
 * Performs any tasks necessary to cleanly shut down the science application.
 * @return Returns true.
 */
class ShutdownHandler : public xmlrpc_c::method {
	public:
		/**
		 * This method is called when the sciapp.shutdown RPC is handled.
		 * @param paramList List of RPC parameters.
		 * @param retvalP Pointer to the return value of the method.
		 */
		void execute(xmlrpc_c::paramList const&,xmlrpc_c::value* const);
};

} // end namespace prc

#endif // _RPC_HANDLER_HPP
