// System Includes
#include <cstdlib>
#include <iostream>
#include <signal.h>

// Local includes
#include "rpc_handler.hpp"
#include "common/server.hpp"

using std::cout;
using std::endl;

using namespace prc;

/**
 * Causes the server to shut down, which will 
 * then cause the main to exit its sleep loop.
 */
void shutdown(int signum) {
	Server::instance()->shutdown();
}

/**
 * Starts the XML-RPC server and waits for requests
 * from the PRC client.
 */
int main(int argc, char** argv) {
	const int SLEEP_DELAY = 5;		// Sleep 5 seconds in while loop in main.
	const int SERVER_PORT = 2082;		// Default science application port is 2082.
	const char* const STDOUT_LOG = "log/sciapp.out.log";
	const char* const STDERR_LOG = "log/sciapp.err.log";

	// Redirect stdout and stderr to the log files.
	freopen(STDOUT_LOG,"w",stdout);
	freopen(STDERR_LOG,"w",stderr);

	// Initialize XML-RPC server.
	Server::instance()->setServerPort(SERVER_PORT);

	// Register RPC handlers.
	Server::instance()->addRPCHandler("sciapp.computeResult", new ComputeResultHandler());
	Server::instance()->addRPCHandler("sciapp.computeSpotCheckResult", new ComputeSpotCheckResultHandler());
	Server::instance()->addRPCHandler("sciapp.shutdown", new ShutdownHandler());

	// Register signal handler
	signal(SIGINT, shutdown);

	// Start the server.
	Server::instance()->start();

	// The server is in its own thread, so just sleep until the science application is terminated.
	cout << "Science Application Ready." << endl;
	while (Server::instance()->isRunning() == true) {
		sleep(SLEEP_DELAY);
	}

	// Stop the server.
	Server::instance()->stop();

	return EXIT_SUCCESS;
}
