#ifndef _SCI_APP_CLIENT_HPP
#define _SCI_APP_CLIENT_HPP

// System includes
#include <cstring>
#include <vector>

// Local includes
#include "common/client.hpp"
#include "common/common.hpp"

using std::string;
using std::vector;

namespace prc {

/**
 * The ScienceApplicationClient is an XML-RPC client with 
 * added functionality that is needed by the science application.
 * This class has methods for saving and retrieving checkpoints.
 * It uses private inheritance from Client because there should never
 * be a reason to access Client's members.  The science application 
 * should _always_ expect the PRC Client to be on 127.0.0.1.  If a 
 * different port number is required, it can be changed by editing
 * the ScienceApplicationClient's constructor.
 */
class ScienceApplicationClient : private Client {
	private:
		static ScienceApplicationClient* clientInstance;
		static const string GET_CHECKPOINT_RPC_NAME;
		static const string SAVE_CHECKPOINT_RPC_NAME;
		bool doCheckpointing;
		/**
 		 * Private constructor because this class is a singleton.
		 */
		ScienceApplicationClient(string addr = "http://127.0.0.1:2081/RPC2") : Client(addr) { }
	public:
		static ScienceApplicationClient* instance();
		void setCheckpointing(bool);
		vector<byte> getCheckpoint();
		bool saveCheckpoint(const vector<byte>&);
};

} // end namespace prc

#endif // _SCI_APP_CLIENT_HPP
