#ifndef _VALIDATOR_CLIENT_HPP
#define _VALIDATOR_CLIENT_HPP

// System includes
#include <cstring>
#include <vector>

// Local includes
#include "common/client.hpp"
#include "common/common.hpp"

using std::string;
using std::vector;

namespace prc {

// Types of validation.
enum ValidationType { SINGLE=0, CANONICAL=1, SPOTCHECK=2 };

// This struct stores all relevant Result information.
struct Result {
	string resultID;		/** The ID of this result. */
	vector<byte> data;		/** The computed result data. */
};

// This struct stores all relevant spot-check information.
struct SpotCheck {
	string resultID;		/** The ID of the client's result. */
	vector<byte> clientData;	/** Result data computed by the client. */
	vector<byte> acceptedData;	/** Accepted result data computed by the project server. */
};

/**
 * The ValidatorClient is an XML-RPC client with 
 * added functionality that is needed by the result validator.
 */
class ValidatorClient : public Client {
	private:
		static ValidatorClient* clientInstance;
		static const string GET_RESULT_RPC_NAME;
		static const string MARK_VALID_RPC_NAME;
		static const string MARK_INVALID_RPC_NAME;
		static const string SELECT_CANONICAL_RPC_NAME;
		static const string GET_ASSOCIATED_WU_RPC_NAME;
		/**
 		 * Private constructor because this class is a singleton.
		 */
		ValidatorClient(string addr = "http://127.0.0.1:2080/RPC2") : Client(addr) { }
	public:
		static ValidatorClient* instance();
		void validateNextResult();
		bool markResultValid(string&);
		bool markResultInvalid(string&);
		bool selectCanonicalResult(string&);
		vector<byte> ValidatorClient::getAssociatedWorkUnit(string&);
};

} // end namespace prc

#endif // _VALIDATOR_CLIENT_HPP
