// System includes
#include <cstdlib>
#include <iostream>

// Local includes
#include "generator.hpp"
#include "work_gen_client.hpp"

using std::cout;
using std::endl;

using namespace prc;

int main(int argc, char** argv) {
	const int RETRY_DELAY = 300;		// Wait 5 minutes before retrying an XML-RPC.

	cout << "Starting Work Unit Generator." << endl;
	WorkUnitGenerator generator;

	// If specified via the command line, set the server address.
	if (argc == 2)
		WorkGeneratorClient::instance()->setServerAddress(argv[1]);

	// Periodically check how many ingress work units the server has.
	// If there are fewer than 5, generate 25 more.
	while (true) {
		size_t numWorkUnits = WorkGeneratorClient::instance()->getNumWorkUnits();
		cout << "The server has " << numWorkUnits << " ingress work units." << endl;
		if (numWorkUnits < 5) {
			cout << "The server has fewer than 5 ingress work units.  Generating 25 more." << endl;
			generator.generateAndSend(25);
		}
		else
			cout << "The server has enough new work units." << endl;

		// Sleep for RETRY_DELAY seconds, then check the server again.
		sleep(RETRY_DELAY);
	}

	return EXIT_SUCCESS;
}
