package edu.wpi.prc.example.java.validator;

import edu.wpi.prc.example.java.common.RPCHandlerException;

/**
 * This class contains the result validator's main.
 * @author James Baldassari
 */
public class Validator {
	/**
	 * Starts the result validator.
	 * @param args Command-line arguments.  The address (host:port) of the project server
	 * can be specified by passing it as a command-line argument.
	 */
	public static void main(String[] args) {
		// If no results found, wait 30 seconds before trying again.
		final int RETRY_DELAY = 30000;

		System.out.println("Starting Example Result Validator.");

		// If specified via the command line, set the server address.
		if (args.length == 2)
			ValidatorClient.instance().setServerAddress(args[1]);

		// Sit in a loop, validating results.
		// If the server did not have any results, wait for RETRY_DELAY/1000 seconds, then check again.
		while (true) {
			try {
				ValidatorClient.instance().validateNextResult();
			} catch (RPCHandlerException e) {
				// There were no results to validate.
				System.out.println("Retrying in " + (RETRY_DELAY / 1000) + " seconds.");
				try {
					Thread.sleep(RETRY_DELAY);
				} catch (InterruptedException e1) {
				}
			}
		}
	}
}
