// System includes
#include <cstdlib>
#include <cstring>
#include <iostream>
#include <sstream>
#include <vector>

// Local includes
#include "common/common.hpp"
#include "generator.hpp"
#include "work_gen_client.hpp"

using std::cout;
using std::endl;
using std::string;
using std::stringstream;
using std::vector;

namespace prc {

/**
 * The default constructor calls recoverState()
 * to get the last work unit that was generated 
 * and initialize the generator so that it resumes 
 * generating in the correct place in the science 
 * data.
 */
WorkUnitGenerator::WorkUnitGenerator() {
	recoverState();
}

/**
 * If the work unit generator were shut down, then started back up, it would 
 * need to determine the last work unit that was generated so that it would 
 * know where to resume generating new ranges.  This method asks the server 
 * for the last work unit that was generated.  If the server throws an 
 * exception, it means that no work units had ever been generated, in 
 * which case the work unit generator is starting for the first time.
 * This method should change the values of some private member variables, 
 * which will be used by the generateWorkUnit() method.
 */
void WorkUnitGenerator::recoverState() {
	try {
		// Get the last work unit.
		// NOTE: WorkUnit is a struct defined in work_gen_client.hpp
		WorkUnit wu = WorkGeneratorClient::instance()->getLastWorkUnit();

		// If control reaches this point it means that a work unit 
		// was received from the server.

		// TODO: Inspect the last work unit to determine how to generate
		// the next one.
		vector<byte>* workUnitData = &(wu.data);

		// TODO: Set some private member variables indicating how the 
		// generator should generate the next work unit.

	} catch (string error) {
		// The server had zero work units.
		// This means that we should start generating
		// our first work unit.

		cout << "recoverState: Initializing the work unit generator for the first work unit." << endl;

		// TODO: Initialize the generator to generate the first work unit.
	}
}

/**
 * Generates the next work unit.
 * NOTE: A "byte" is simply an unsigned char.  It is a typedef defined 
 * in common/common.hpp.
 * @return Returns a pointer to a vector<byte> that contains the work unit data.
 */
vector<byte>* WorkUnitGenerator::generateWorkUnit() {
	// Allocate a new byte vector.
	vector<byte>* wu = new vector<byte>;

	// TODO: Generate the next work unit using some private member
	// variables to determine what data should be included 
	// in this work unit.
	
	// TODO: Convert the work unit into a byte vector and insert it
	// into "wu"

	// TODO: Update the private member variables so that they will 
	// be properly initialized for generating the next work 
	//.unit.

	// Return the generated work unit.
	return wu;
}

/**
 * Generates and sends a given number of work units to the project server.
 * @param numWorkUnits The number of work units to generate and send.
 * @return Returns the number of clients that are blocked waiting for work units.
 */
size_t WorkUnitGenerator::generateAndSend(int numWorkUnits) {
	cout << "Generating " << numWorkUnits << " new work units, then sending them to the server." << endl;
	size_t numWaiting = 0;

	// Generate and send the specified number of work units.
	for (int ii = 0; ii < numWorkUnits; ii++) {
		// Generate the next work unit data.
		vector<byte>* workUnit = generateWorkUnit();

		// TODO: If you do not need to specify the priority, point value, or 
		// expiration time of the work unit, you can choose to use a different 
		// sendWorkUnit method from WorkGeneratorClient.
		
		// NOTE: There are several variants of the "WorkGeneratorClient.sendWorkUnit" method.
		// The simplest case only takes the data byte array.  This example 
		// specifies several parameters including data, priority, point value, 
		// and expiration time.
		// See WorkGeneratorClient for other versions of the RPC.

		// Send the work unit to the server.
		numWaiting = WorkGeneratorClient::instance()->sendWorkUnit(
			*workUnit,
			DEFAULT_PRIORITY,
			DEFAULT_POINTS,
			DEFAULT_EXPIRATION_TIME);

		// Here are some other methods that can be used:

		// WorkGeneratorClient::instance()->sendWorkUnit(
		//	workUnitID, workUnit, DEFAULT_PRIORITY, 
		//	DEFAULT_POINTS, DEFAULT_EXPIRATION_TIME);

		// WorkGeneratorClient::instance()->sendWorkUnit(
		//	workUnitID, workUnit, 
		//	DEFAULT_PRIORITY, DEFAULT_POINTS);

		// WorkGeneratorClient::instance()->sendWorkUnit(
		//	workUnit, DEFAULT_PRIORITY, DEFAULT_POINTS);

		// WorkGeneratorClient::instance()->sendWorkUnit(workUnit);

		// The work unit has been sent, so delete it.
		delete workUnit;
	}

	// Return the number of users waiting for work units.
	return numWaiting;
}

} // end namespace prc
