package edu.wpi.prc.templates.java.generator;

/**
 * The Generator initializes the WorkGeneratorClient client and
 * periodically queries the server for the number of ingress
 * work units it has.  If the server does not have enough, 
 * the WorkUnitGenerator is used to generate and send more.
 * @author James Baldassari
 */
public class Generator {
	private static final int RETRY_DELAY = 30000;	// Wait 30 seconds before checking the server again.
	public static void main(String[] args) {
		System.out.println("Starting Work Unit Generator.");
		WorkUnitGenerator wuGenerator = new WorkUnitGenerator();
		
		// If specified via the command line, set the server address.
		if (args.length == 1)
			WorkGeneratorClient.instance().setServerAddress(args[0]);
		
		// Periodically check how many ingress work units the server has.
		// If there are fewer than 5, generate 25 more.
		while (true) {
			// Get the number of work units the server has.
			int numWorkUnits = WorkGeneratorClient.instance().getNumWorkUnits();
			System.out.println("The server has " + numWorkUnits + " ingress work units.");
			
			// TODO: You may want to change the "low-water" mark for work units from 5 to something higher.
			if (numWorkUnits < 5) {
				// TODO: You also may want to generate more than 25 work units at a time.
				System.out.println("The server has fewer than 5 ingress work units.  Generating 25 more.");
				wuGenerator.generateAndSend(25);
			}
			
			else {
				System.out.println("The server has enough work units.");
				try {
					Thread.sleep(RETRY_DELAY);
				} catch (InterruptedException e) {

				}
			}
		}
	}
}
