package edu.wpi.prc.templates.java.science;

/**
 * The ComputeThread runs the science algorithm (from ScienceDataProcessor)
 * in a low-priority thread.
 * @author James Baldassari
 */
public class ComputeThread extends Thread {
	private byte[] workUnit = null;
	private byte[] result = null;
	
	/**
	 * Constructor that takes the work unit data.
	 * @param workUnit The work unit data to be processed.
	 */
	public ComputeThread(byte[] workUnit) {
		this.workUnit = workUnit;
		this.setPriority(Thread.MIN_PRIORITY);	// This thread should have the lowest possible priority.
		this.setName("Compute Thread");
	}
	
	/**
	 * Invokes the ScienceDataProcessor to compute the result for the 
	 * current work unit.
	 */
	public void run() {
		ScienceDataProcessor processor = new ScienceDataProcessor(workUnit);
		result = processor.computeResult();
	}
	
	/**
	 * @return Returns the result computed by the science algorithm.
	 */
	public byte[] getResult() {
		return result;
	}
}
