package edu.wpi.prc.templates.java.validator;

import java.util.Vector;

/**
 * The ResultValidator class contains project-specific code
 * used to validate work units and spot-checks, and to select
 * the canonical result for a work unit.
 * @author James Baldassari
 */
public class ResultValidator {
	/**
	 * Determines whether a single result was valid.
	 * @param r The result to validate.
	 * @return Returns true if the result is valid, false otherwise.
	 */
	public boolean validateSingleResult(Result r) {
		boolean isValid = true;
		System.out.println("Result Validator: Validating result " + r.getResultID());
		
		// Inspect the result data to determine whether it is valid.
		byte[] resultData = r.getData();
		
		// If necessary, it is possible to get the work unit from which this result was computed.
		// byte[] workUnitData = ValidatorClient.instance().getAssociatedWorkUnit(r.getResultID());
		
		return isValid;
	}
	
	/**
	 * Selects a canonical result from a set of results.
	 * @param results The set of valid results for a work unit.
	 * @return Returns the result ID of the canonical result that was selected.
	 */
	public String selectCanonicalResult(Vector<Result> results) {
		// Initialize the canonical result to the first result in the Vector
		// so that we are sure to return some valid ID.
		String canonicalResult = results.get(0).getResultID();
		
		// If necessary, it is possible to get the work unit from which this result was computed.
		// The work unit will be the same for all results in the Vector.
		// byte[] workUnitData = ValidatorClient.instance().getAssociatedWorkUnit(canonicalResult);
		
		// Loop through, examining the results, and choose one to be the canonical result. 
		for (Result r : results) {
			// Examine the result data.
			byte[] resultData = r.getData();
			
			// If this result should be the canonical result, set canonicalResult to this result's ID.
			canonicalResult = r.getResultID();
		}
		
		// Return the ID of the canonical result.
		System.out.println("Result Validator: Selecting canonical result " + canonicalResult);
		return canonicalResult;
	}
	
	/**
	 * Determines whether a spot-check was valid.
	 * @param check The spot-check to validate.
	 * @return Returns true if the spot-check is valid, false otherwise.
	 */
	public boolean validateSpotCheck(SpotCheck check) {
		boolean isValid = true;
		System.out.println("Result Validator: Validating spot-check " + check.getResultID());		
		
		// The client data is the result data computed by a volunteer's computer.
		byte[] clientData = check.getClientData();
		
		// The accepted data is the result data that was computed by the project server.
		byte[] acceptedData = check.getAcceptedData();
		
		// If necessary, it is possible to get the work unit from which this spot-check was computed.
		// byte[] workUnitData = ValidatorClient.instance().getAssociatedWorkUnit(check.getResultID());
		
		// Compare the two byte arrays using some algorithm, and determine 
		// whether the spot-check was valid. 
		
		return isValid;
	}
}
