package edu.wpi.prc.client;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import edu.wpi.prc.persistence.SysLogger;

/**
 * Processes all science application XML-RPC requests.
 * Acts as a proxy between the science application and the project server.
 * @author James Baldassari
 */
public class ClientRPCHandler {
	private PRCClient client;
	
	/**
	 * Constructor.
	 * @param client The PRCClient that is using this ClientRPCHandler.
	 */
	public ClientRPCHandler(PRCClient client) {
		this.client = client;
	}
	
	/**
	 * Saves checkpoint data to disk.
	 * @param data The data to save.
	 * @return Returns true if the save was successful, false otherwise.
	 */
	public boolean saveCheckpoint(byte[] data) {
		boolean success = false;
		try {
			OutputStream os = new FileOutputStream(PRCClient.CHECKPOINT_PATH);
			os.write(data);
			os.flush();
			os.close();
			success = true;
		} catch (FileNotFoundException e) {
			System.out.println("ERROR: Could not open checkpoint file " + PRCClient.CHECKPOINT_PATH);
			SysLogger.println("ERROR: Could not open checkpoint file " + PRCClient.CHECKPOINT_PATH);
		} catch (IOException e) {
			System.out.println("ERROR: Could not write checkpoint file " + PRCClient.CHECKPOINT_PATH);
			SysLogger.println("ERROR: Could not write checkpoint file " + PRCClient.CHECKPOINT_PATH);			
		}
		return success;
	}
	
	/**
	 * Retrieves saved checkpoint data from disk.
	 * @return Returns true if checkpoint retrieval was successful, false otherwise.
	 */
	public byte[] getCheckpoint() {
		byte[] checkpoint = new byte[0];
		try {
			InputStream is = new FileInputStream(PRCClient.CHECKPOINT_PATH);
			checkpoint = new byte[is.available()];
			is.read(checkpoint);
		} catch (FileNotFoundException e) {
			SysLogger.println("No checkpoint file found (" + PRCClient.CHECKPOINT_PATH + ").");
		} catch (IOException e) {
			System.out.println("ERROR: Could not read checkpoint file " + PRCClient.CHECKPOINT_PATH);
			SysLogger.println("ERROR: Could not read checkpoint file " + PRCClient.CHECKPOINT_PATH);			
		}
		
		return checkpoint;
	}
	
	/**
	 * Shuts down the PRCClient.
	 * @return Returns true.
	 */
	public boolean shutdown() {
		client.shutdown();
		return true;
	}
}
