package edu.wpi.prc.project.test;

import org.hibernate.HibernateException;

import edu.wpi.prc.persistence.DatabaseType;
import edu.wpi.prc.persistence.HibernateController;
import edu.wpi.prc.project.Project;
import edu.wpi.prc.project.SpotCheckFailureAction;
import junit.framework.TestCase;

/**
 * Tests persisting a Project to the database.
 * @author James Baldassari
 */
public class ProjectTest extends TestCase {
	public ProjectTest(String name) {
		super(name);
		
		HibernateController.instance().initializeForUnitTests();
		HibernateController.instance().exportSchema();
	}
	
	/**
	 * Test saving a Project to the database.
	 */
	public void testSave() {
		Project p = Project.instance();
		p.setDbURI("data/prc");
		p.setDbType(DatabaseType.HSQLDB);
		p.setMinNumResults(3);
		p.setName("JDB PRC Thesis");
		p.setScienceApp("MyScienceApp");
		p.setSpotCheckProbability(0.01f);
		p.setMinNumSpotChecks(1);
		p.setSpotCheckFailureAction(SpotCheckFailureAction.BanUser);
		p.setServerURI("cstag04.cs.wpi.edu:2080");
		
		// Save the Project to the database.
		try {
			p.saveOrUpdate();
		} catch (HibernateException e) {

		}
		
		// Verity that the name is correct.
		assertTrue(Project.instance().getName().equals("JDB PRC Thesis"));
		
		// Shut down Hibernate so that the data will persist in HSQLDB.
		HibernateController.instance().shutdown();
		
		// Reinitialize Hibernate.
		HibernateController.instance().initializeForUnitTests();
		
		// Retrieve Project from the database (requires Project to have already been saved)
		Project p2 = Project.instance();
		
		// Verify that the name is correct.
		assertTrue(p2.getName().equals("JDB PRC Thesis"));
		assertTrue(p2.getSpotCheckFailureAction() == SpotCheckFailureAction.BanUser);
		assertTrue(p2.getDefaultValidator() == true);
		assertTrue(p2.getServerURI().equals("cstag04.cs.wpi.edu:2080"));
	}
}
