/**
 * 
 */
package edu.wpi.prc.tools.builder.gui;

import javax.swing.JFrame;

import org.hibernate.HibernateException;

import edu.wpi.prc.persistence.HibernateController;
import edu.wpi.prc.project.MissingConfigurationFileException;
import edu.wpi.prc.project.Project;
import edu.wpi.prc.tools.builder.ProjectBuilder;

/**
 * Attempts to connect to the project database.  During the connection 
 * process, a window is displayed notifying the user that the Project 
 * Builder is busy trying to connect to the project database.
 * @author James Baldassari
 */
public class ConnectionFormThread extends Thread {
	private DatabaseInitializerForm parentForm = null;
	private JFrame connectingForm = null;
	private Project project = null;

	/**
	 * Constructor that takes a reference to the this thread's parent form.
	 */
	public ConnectionFormThread(DatabaseInitializerForm parentForm) {
		this.parentForm = parentForm;
		connectingForm = new ConnectingForm();
	}
	
	/**
	 * Makes the "connecting" window appear, and attempts to connect to the project database.
	 */
	public void run() {
		connectingForm.setVisible(true);
		try {
			HibernateController.instance().shutdown();
			HibernateController.instance().initialize(ProjectBuilder.SERVER_CONFIG_FILE_PATH);
			parentForm.completeDatabaseConnection();
		} catch (MissingConfigurationFileException e) {
			String message = "The project configuration file was not found.  Please verify that\n";
			message += "the \"project_server.properties\" file is in the \"cfg\" directory.";
			ProjectBuilder.instance().showErrorDialog(parentForm, message);
			parentForm.setVisible(true);
		} catch (HibernateException e) {
			String message = "The Database connection failed.  If you are using an HSQLDB\n";
			message += "database, verify that the database files exist in the location\n";
			message += "specified in your project configuration file.  If you are not\n";
			message += "using an HSQLDB database, verify that your database is\n";
			message += "configured correctly, that it is accepting connections, and\n";
			message += "that this computer can connect to the computer hosting\n";
			message += "the database.";
			ProjectBuilder.instance().showErrorDialog(parentForm, message);
			parentForm.setVisible(true);
		}
	}
	
	/**
	 * Destroys the "connecting" form.
	 */
	public void cleanup() {
		connectingForm.dispose();
	}
	
	/**
	 * Reads the project that is stored in the database.
	 * @return The project that was retrieved from the database.
	 */
	public Project getProject() throws HibernateException {
		try {
			project = Project.readFromDatabase();
		} catch (HibernateException e) {
			project = Project.createProject();
			throw e;
		}
		return project;
	}
	
	/**
	 * @return Returns the "connecting" form.
	 */
	public JFrame getForm() {
		return connectingForm;
	}
}
