package edu.wpi.prc.util.password;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

/**
 * This class reads a password from the command prompt, but does not display it in cleartext.
 * This is basically a hack that erases the characters after the user types them, but unfortunately
 * Java has no support for reading passwords.  This class will be updated after JDK 6 is released, 
 * which will have this functionality.
 * NOTE: This implementation inspired by http://java.sun.com/developer/technicalArticles/Security/pwordmask/.
 * @author James Baldassari
 */
public class PasswordReader {
	private BufferedReader br = null;
	private PasswordEraserThread eraser = null;
	private String password = null;
	
	/**
	 * Default constructor.
	 */
	public PasswordReader() {
		br = new BufferedReader(new InputStreamReader(System.in));
	}
	
	/**
	 * Reads the password from the console, replacing all characters with asterisks.
	 * @return Returns the password entered by the user.
	 * @throws IOException
	 */
	public String readLine() throws IOException {
		eraser = new PasswordEraserThread("Enter project password (will not be echoed): ");
		eraser.start();
		password = br.readLine();
		eraser.stopMasking();
		eraser = null;
		return password;
	}
	
	/**
	 * @return Returns the password entered by the user.
	 */
	public String getPassword() {
		return password;
	}
}
